package net.wanji.utc.task;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.common.utils.tool.LocalDateTimeUtil;
import net.wanji.databus.dao.entity.SignalCommandLogPO;
import net.wanji.databus.dao.entity.SignalCommandPO;
import net.wanji.databus.dao.mapper.SignalCommandLogPOMapper;
import net.wanji.databus.dao.mapper.SignalCommandPOMapper;
import net.wanji.databus.vo.ControlCommandVO;
import net.wanji.databus.vo.LightsStatusVO2;
import net.wanji.utc.cache.CrossInfoCache;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.service.control.ControlCommandService;
import net.wanji.utc.service.control.ControlCommandStrategyService;
import net.wanji.utc.util.StringUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.EnableAsync;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 定时同步方案等静态信息
 *
 * @author zhouleilei
 * @date 2024/12/30 14:01
 */
@Component
@Slf4j
@EnableAsync
@SuppressWarnings("all")
public class CommandFaildTask {
    @Resource
    private SignalCommandPOMapper signalCommandPOMapper;
    @Resource
    private ControlCommandStrategyService controlCommandStrategyService;

    @Resource(name = "hkControlCommandService")
    ControlCommandService hkControlCommandService;
    @Resource(name = "wanJiControlCommandService")
    ControlCommandService wanJiControlCommandService;
    @Resource
    private CrossInfoCache crossInfoCache;
    @Resource
    private SignalCommandLogPOMapper signalCommandLogPOMapper;

    @Scheduled(cron = "0/5 * * * * ?")
    @Async
    public void commandFaildTask() throws Exception {
        LambdaQueryWrapper<SignalCommandPO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(SignalCommandPO::getCommandResult, 2);
        List<SignalCommandPO> crossInfoPOList = signalCommandPOMapper.selectList(lambdaQueryWrapper);
        if (ObjectUtil.isNotEmpty(crossInfoPOList)) {
            for (SignalCommandPO signalCommandPO : crossInfoPOList) {
                String crossId = signalCommandPO.getCrossId();
                Integer commandType = signalCommandPO.getCommandType();
                Integer updateCount = signalCommandPO.getUpdateCount();
                if (ObjectUtil.isNull(updateCount)){
                    updateCount = 0;
                }
                try {
                    //处理数据
                    handlerCommand(crossId, commandType,updateCount);
                    log.info("定时任务自动恢复执行成功(不保证恢复成功),crossId={},commandType={},updateCount={}",crossId,commandType,updateCount);
                } catch (Exception e) {
                    log.error("定时任务自动恢复报错,crossId={},commandType={},updateCount={}",crossId,commandType,updateCount);
                    throw new RuntimeException(e);
                }
            }
        }
    }

    /**
     * @return void
     * @Description 处理数据
     * @Param crossId        路口编号
     * @Param commandType    控制模式
     **/
    private void handlerCommand(String crossId, Integer commandType,int updateCount) {
        switch (commandType) {
            //1-步进控制/恢复(公用);2-恢复时间表(公用);3-相位锁定/解锁(公用);4-临时方案下发(公用);5-下发命令失败，程序下发恢复指令；10-查询环图失败记录',
            case 1:
                //恢复步进
                recoverStepControlStrategy(crossId,updateCount,commandType);
                break;
            case 2:
                //恢复时间表
                recoverScheduleStrategy(crossId,updateCount,commandType);
                break;
            case 3:
                //相位解锁
                recoverLockControlStrategy(crossId,updateCount,commandType);
                break;
            case 4:
                //恢复 临时方案下发
                recoverScheduleStrategy(crossId, updateCount,commandType);
                break;
            default:
                log.error("commandType有误,crossId={},commandType={}", crossId, commandType);
                break;

        }
    }

    /**
     * @return void
     * @Description 恢复步进
     * @Param crossId
     **/
    private void recoverStepControlStrategy(String crossId,int updateCount,int commandType) {
        String now = LocalDateTimeUtil.formatNow(LocalDateTimeUtil.TIMEFORMATTER);
        JsonViewObject jsonViewObject;
        try {
            String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
            if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
                jsonViewObject = hkControlCommandService.stepControl(crossId, 0, 0);
            } else {
                jsonViewObject = wanJiControlCommandService.stepControl(crossId, 0, 0);
            }
        } catch (Exception e) {
            jsonViewObject = JsonViewObject.newInstance().fail(e.getMessage());
        }
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("crossId", crossId);
        jsonObject.put("command", 0);
        jsonObject.put("stepNum", 0);
        //插入命令操作日志
        insertCommandLog(now, jsonViewObject, jsonObject,commandType,updateCount);
    }

    /**
     * @return void
     * @Description 恢复时间表
     * @Param crossId
     **/

    private void recoverScheduleStrategy(String crossId,int updateCount,int commandType) {
        try {
            String now = LocalDateTimeUtil.formatNow(LocalDateTimeUtil.TIMEFORMATTER);
            JsonViewObject jsonViewObject = JsonViewObject.newInstance();
            String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(crossId);
            if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
                jsonViewObject = hkControlCommandService.recoverSchedule(crossId);
            } else {
                jsonViewObject = wanJiControlCommandService.recoverSchedule(crossId);
            }
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("crossId",crossId);
            //插入命令操作日志
            insertCommandLog(now,jsonViewObject, jsonObject,commandType,updateCount);
        } catch (Exception e) {
            log.error("定时任务 恢复时间表出错,crossId={}",crossId);
            throw new RuntimeException(e);
        }
    }

    /**
     * @return void
     * @Description 相位解锁
     * @Param crossId
     **/

    private void recoverLockControlStrategy(String crossId,int updateCount,int commandType) {
        String now = LocalDateTimeUtil.formatNow(LocalDateTimeUtil.TIMEFORMATTER);
        JsonViewObject jsonViewObject;
        ControlCommandVO commandVO = new ControlCommandVO();
        commandVO.setCrossCode(crossId);
        commandVO.setCommand(0);
        commandVO.setDuration(0);
        try {
            String manufacturerIdCode = crossInfoCache.getManufacturerCodeByCrossId(commandVO.getCrossCode());
            if (StringUtils.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerIdCode)) {
                jsonViewObject = hkControlCommandService.lockControl(commandVO);
            } else {
                jsonViewObject = wanJiControlCommandService.lockControl(commandVO);
            }
        } catch (Exception e) {
            jsonViewObject = JsonViewObject.newInstance().fail(e.getMessage());
        }
        JSONObject jsonObject = JSONObject.parseObject(JSON.toJSONString(commandVO));
        jsonObject.put("crossId",commandVO.getCrossCode());
        //插入命令操作日志
        insertCommandLog(now,jsonViewObject, jsonObject,commandType,updateCount);
    }

    /**
     * @return void
     * @Description 插入命令日志
     * @Param nowTime 请求时间戳
     * @Param jsonViewObject 操作结果
     * @Param jsonObject     参数
     **/
    public void insertCommandLog(String queryTime, JsonViewObject jsonViewObject, JSONObject jsonObject, int commandType,int updateCount) {
        updateCount = updateCount+1;
        SignalCommandLogPO signalCommandLogPO = new SignalCommandLogPO();
        signalCommandLogPO.setCrossId(jsonObject.getString("crossId"));
        signalCommandLogPO.setDataInfo(jsonObject.toJSONString());
        signalCommandLogPO.setCommandType(5);
        signalCommandLogPO.setQueryTime(queryTime);
        signalCommandLogPO.setUpdateCount(updateCount);
        Integer code = jsonViewObject.getCode();
        String ip = "localhost";
        signalCommandLogPO.setIp(ip);
        if (code == 200) {
            signalCommandLogPO.setCommandResult(1);
            signalCommandLogPO.setUpdateCount(0);
        } else {
            if (updateCount >= 24){
                //如果两分钟更新失败，则将状态设置为3
                signalCommandLogPO.setCommandResult(3);
            }else {
                signalCommandLogPO.setCommandResult(2);

            }
        }
        signalCommandLogPO.setResultMessage(jsonViewObject.getMessage());

        ConcurrentHashMap<String, List<LightsStatusVO2>> produceListMap = SignalStatusTask.produceListMap;
        if (!produceListMap.isEmpty()) {
            List<LightsStatusVO2> list = produceListMap.get(jsonObject.get("crossId"));
            if (ObjectUtil.isNotEmpty(list)) {
                LightsStatusVO2 lightsStatusVO2 = list.get(0);
                String runMode = lightsStatusVO2.getRunMode();
                String lampTime = lightsStatusVO2.getTimeStamp();
                //时间转换
//                lampTime = LocalDateTimeUtil.formatTimeStamp(Long.valueOf(lampTime), LocalDateTimeUtil.TIMEFORMATTER);
                signalCommandLogPO.setRunMode(runMode);
                signalCommandLogPO.setLampTime(lampTime);
            }
        }
        //插入日志表
        signalCommandLogPOMapper.insert(signalCommandLogPO);
        SignalCommandPO signalCommandPO = new SignalCommandPO();
        BeanUtil.copyProperties(signalCommandLogPO, signalCommandPO);
        signalCommandPO.setCommandType(commandType);
        //更新状态表
        LambdaQueryWrapper<SignalCommandPO> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(SignalCommandPO::getCrossId,jsonObject.getString("crossId"));
        signalCommandPOMapper.update(signalCommandPO,lambdaQueryWrapper);
    }

}
